@@ -1,33 +1,12 @@
 /*
- * Copyright (c) 2009-2015 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2009-2015 Jason Mehrens. All Rights Reserved.
+ * Copyright (c) 2009, 2019 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2018 Jason Mehrens. All Rights Reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Distribution License v. 1.0, which is available at
+ * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- *   - Redistributions of source code must retain the above copyright
- *     notice, this list of conditions and the following disclaimer.
- *
- *   - Redistributions in binary form must reproduce the above copyright
- *     notice, this list of conditions and the following disclaimer in the
- *     documentation and/or other materials provided with the distribution.
- *
- *   - Neither the name of Oracle nor the names of its
- *     contributors may be used to endorse or promote products derived
- *     from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
- * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
- * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 import com.sun.mail.util.logging.CollectorFormatter;
@@ -49,7 +28,7 @@ import javax.mail.internet.InternetAddress;
 /**
  * Demo for the different configurations for the MailHandler. If the logging
  * properties file or class is not specified then this demo will apply some
- * default settings to store emails in the users temp dir.
+ * default settings to store emails in the user's temp directory.
  *
  * @author Jason Mehrens
  */
@@ -65,18 +44,22 @@ public class MailHandlerDemo {
     private static final Logger LOGGER = Logger.getLogger(CLASS_NAME);
 
     /**
+     * Runs the demo.
+     *
      * @param args the command line arguments
+     * @throws IOException if there is a problem.
      */
-    public static void main(String[] args) {
+    public static void main(String[] args) throws IOException {
         List<String> l = Arrays.asList(args);
         if (l.contains("/?") || l.contains("-?") || l.contains("-help")) {
             LOGGER.info("Usage: java MailHandlerDemo "
-                    + "[[-all] | [-body] | [-debug] | [-low] | [-simple] "
-                    + "| [-pushlevel] | [-pushfilter] | [-pushnormal]"
-                    + "| [-pushonly]] "
+                    + "[[-all] | [-body] | [-custom] | [-debug] | [-low] "
+                    + "| [-simple] | [-pushlevel] | [-pushfilter] "
+                    + "| [-pushnormal] | [-pushonly]] "
                     + "\n\n"
                     + "-all\t\t: Execute all demos.\n"
                     + "-body\t\t: An email with all records and only a body.\n"
+                    + "-custom\t\t: An email with attachments and dynamic names.\n"
                     + "-debug\t\t: Output basic debug information about the JVM "
                     + "and log configuration.\n"
                     + "-low\t\t: Generates multiple emails due to low capacity."
@@ -97,10 +80,10 @@ public class MailHandlerDemo {
                     + "priority emails when the push level is triggered and "
                     + "normal priority when flushed.\n");
         } else {
-            init(l); //may create log messages.
+            final boolean debug = init(l); //may create log messages.
             try {
                 LOGGER.log(Level.FINEST, "This is the finest part of the demo.",
-                        new MessagingException("Fake JavaMail issue."));
+                        new MessagingException("Fake Jakarta Mail issue."));
                 LOGGER.log(Level.FINER, "This is the finer part of the demo.",
                         new NullPointerException("Fake bug."));
                 LOGGER.log(Level.FINE, "This is the fine part of the demo.");
@@ -122,6 +105,11 @@ public class MailHandlerDemo {
             } finally {
                 closeHandlers();
             }
+
+            //Force parse errors.  This does have side effects.
+            if (debug && getConfigLocation() != null) {
+                LogManager.getLogManager().readConfiguration();
+            }
         }
     }
 
@@ -144,16 +132,24 @@ public class MailHandlerDemo {
         }
 
         try {
+            err.println(prefix + ": java.version="
+                    + System.getProperty("java.version"));
             err.println(prefix + ": LOGGER=" + LOGGER.getLevel());
-            err.println(prefix + ": JVM id " + ManagementFactory.getRuntimeMXBean().getName());
-            err.println(prefix + ": java.security.debug=" + System.getProperty("java.security.debug"));
+            err.println(prefix + ": JVM id "
+                    + ManagementFactory.getRuntimeMXBean().getName());
+            err.println(prefix + ": java.security.debug="
+                    + System.getProperty("java.security.debug"));
             SecurityManager sm = System.getSecurityManager();
             if (sm != null) {
-                err.println(prefix + ": SecurityManager.class=" + sm.getClass().getName());
+                err.println(prefix + ": SecurityManager.class="
+                        + sm.getClass().getName());
+                err.println(prefix + ": SecurityManager classLoader="
+                        + toString(sm.getClass().getClassLoader()));
                 err.println(prefix + ": SecurityManager.toString=" + sm);
             } else {
                 err.println(prefix + ": SecurityManager.class=null");
                 err.println(prefix + ": SecurityManager.toString=null");
+                err.println(prefix + ": SecurityManager classLoader=null");
             }
 
             String policy = System.getProperty("java.security.policy");
@@ -179,17 +175,23 @@ public class MailHandlerDemo {
                 err.println(prefix + ": canRead=" + f.canRead());
                 err.println(prefix + ": lastModified="
                         + new java.util.Date(f.lastModified()));
-                //Force any errors. This is only safe if key is present.
-                manager.readConfiguration();
             } else {
-                err.println(prefix + ": " + key + " is not set as a system property.");
+                err.println(prefix + ": " + key
+                        + " is not set as a system property.");
             }
-            err.println(prefix + ": LogManager.class=" + manager.getClass().getName());
+            err.println(prefix + ": LogManager.class="
+                    + manager.getClass().getName());
+            err.println(prefix + ": LogManager classLoader="
+                    + toString(manager.getClass().getClassLoader()));
             err.println(prefix + ": LogManager.toString=" + manager);
-            err.println(prefix + ": MailHandler classLoader=" + MailHandler.class.getClassLoader());
-            err.println(prefix + ": Context ClassLoader=" + Thread.currentThread().getContextClassLoader());
-            err.println(prefix + ": Session ClassLoader=" + Session.class.getClassLoader());
-            err.println(prefix + ": DataHandler ClassLoader=" + DataHandler.class.getClassLoader());
+            err.println(prefix + ": MailHandler classLoader="
+                    + toString(MailHandler.class.getClassLoader()));
+            err.println(prefix + ": Context ClassLoader="
+                    + toString(Thread.currentThread().getContextClassLoader()));
+            err.println(prefix + ": Session ClassLoader="
+                    + toString(Session.class.getClassLoader()));
+            err.println(prefix + ": DataHandler ClassLoader="
+                    + toString(DataHandler.class.getClassLoader()));
 
             final String p = MailHandler.class.getName();
             key = p.concat(".mail.to");
@@ -236,6 +238,22 @@ public class MailHandlerDemo {
         err.flush();
     }
 
+    /**
+     * Gets the class loader list.
+     *
+     * @param cl the class loader or null.
+     * @return the class loader list.
+     */
+    private static String toString(ClassLoader cl) {
+        StringBuilder buf = new StringBuilder();
+        buf.append(cl);
+        while (cl != null) {
+            cl = cl.getParent();
+            buf.append("<-").append(cl);
+        }
+        return buf.toString();
+    }
+
     /**
      * Gets a formatting string describing the given handler.
      *
@@ -295,6 +313,8 @@ public class MailHandlerDemo {
             err.print(prefix + ": ");
             error.printStackTrace(err);
         }
+
+        buf.append(", ").append(toString(h.getClass().getClassLoader()));
         return buf.toString();
     }
 
@@ -399,9 +419,8 @@ public class MailHandlerDemo {
      * com.sun.mail.util.logging.MailHandler.level=ALL
      * java.util.logging.MemoryHandler.level=ALL
      * java.util.logging.MemoryHandler.push=WARNING
-     * com.sun.mail.util.logging.MailHandler.subject=Push on MessagingException demo
-     * com.sun.mail.util.logging.MailHandler.pushLevel=ALL
-     * com.sun.mail.util.logging.MailHandler.pushFilter=MailHandlerDemo$MessageErrorsFilter
+     * com.sun.mail.util.logging.MailHandler.subject=Push only demo
+     * com.sun.mail.util.logging.MailHandler.pushLevel=WARNING
      * ##
      * </code>
      */
@@ -500,8 +519,9 @@ public class MailHandlerDemo {
      * Sets up the demos that will run.
      *
      * @param l the list of arguments.
+     * @return true if debug is on.
      */
-    private static void init(List<String> l) {
+    private static boolean init(List<String> l) {
         l = new ArrayList<String>(l);
         Session session = Session.getInstance(System.getProperties());
         boolean all = l.remove("-all") || l.isEmpty();
@@ -538,7 +558,8 @@ public class MailHandlerDemo {
         }
 
         boolean fallback = applyFallbackSettings();
-        if (l.remove("-debug") || session.getDebug()) {
+        boolean debug = l.remove("-debug") || session.getDebug();
+        if (debug) {
             checkConfig(CLASS_NAME, session.getDebugOut());
         }
 
@@ -549,6 +570,7 @@ public class MailHandlerDemo {
         if (fallback) {
             LOGGER.info("Check your user temp dir for output.");
         }
+        return debug;
     }
 
     /**
@@ -614,4 +636,12 @@ public class MailHandlerDemo {
         }
         return file;
     }
+
+    /**
+     * No objects are allowed.
+     * @throws IllegalAccessException always.
+     */
+    private MailHandlerDemo() throws IllegalAccessException {
+        throw new IllegalAccessException();
+    }
 }
