@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2019 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2018 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://oss.oracle.com/licenses/CDDL+GPL-1.1
- * or LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package com.sun.mail.imap;
@@ -182,10 +158,52 @@ public class IMAPInputStream extends InputStream {
 	bufpos = ba.getStart();
 	int n = ba.getCount();    // will be zero, if all data has been
 				  // consumed from the server.
-	// if we got less than we asked for, this is the last buffer of data
-	lastBuffer = n < cnt;
+
+	int origin = b != null ? b.getOrigin() : pos;
+	if (origin < 0) {
+	    /*
+	     * Some versions of Exchange will return the entire message
+	     * body even though we only ask for a chunk, and the returned
+	     * data won't specify an "origin".  If this happens, and we
+	     * get more data than we asked for, assume it's the entire
+	     * message body.
+	     */
+	    if (pos == 0) {
+		/*
+		 * If we got more or less than we asked for,
+		 * this is the last buffer of data.
+		 */
+		lastBuffer = n != cnt;
+	    } else {
+		/*
+		 * We asked for data NOT starting at the beginning,
+		 * but we got back data starting at the beginning.
+		 * Possibly we could extract the needed data from
+		 * some part of the data we got back, but really this
+		 * should never happen so we just assume something is
+		 * broken and terminate the data here.
+		 */
+		n = 0;
+		lastBuffer = true;
+	    }
+	} else if (origin == pos) {
+	    /*
+	     * If we got less than we asked for,
+	     * this is the last buffer of data.
+	     */
+	    lastBuffer = n < cnt;
+	} else {
+	    /*
+	     * We got back data that doesn't match the request.
+	     * Just terminate the data here.
+	     */
+	    n = 0;
+	    lastBuffer = true;
+	}
+
 	bufcount = bufpos + n;
 	pos += n;
+
     }
 
     /**
