@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2018 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2015 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package com.sun.mail.imap;
@@ -48,7 +24,7 @@ import javax.mail.*;
 import javax.mail.internet.*;
 import javax.activation.*;
 
-import com.sun.mail.util.*;
+import com.sun.mail.util.ReadableMime;
 import com.sun.mail.iap.*;
 import com.sun.mail.imap.protocol.*;
 
@@ -91,7 +67,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     protected Map<String, Object> items;		// Map<String,Object>
 
     private Date receivedDate;		// INTERNALDATE
-    private int size = -1;		// RFC822.SIZE
+    private long size = -1;		// RFC822.SIZE
 
     private Boolean peek;		// use BODY.PEEK when fetching content?
 
@@ -125,7 +101,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
      * Could this somehow be included in the InternetHeaders object ??
      */
     private Hashtable<String, String> loadedHeaders
-	    = new Hashtable<String, String>(1);
+	    = new Hashtable<>(1);
 
     // This is our Envelope
     static final String EnvelopeCmd = "ENVELOPE INTERNALDATE RFC822.SIZE";
@@ -210,6 +186,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
      * Wrapper around the protected method Message.setMessageNumber() to 
      * make that method accessible to IMAPFolder.
      */
+    @Override
     protected void setMessageNumber(int msgnum) {
 	super.setMessageNumber(msgnum);
     }
@@ -268,6 +245,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     }
 
     // expose to MessageCache
+    @Override
     protected void setExpunged(boolean set) {
 	super.setExpunged(set);
     }
@@ -315,6 +293,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the "From" attribute.
      */
+    @Override
     public Address[] getFrom() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -334,10 +313,12 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return aaclone(a);
     }
 
+    @Override
     public void setFrom(Address address) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
 
+    @Override
     public void addFrom(Address[] addresses) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
@@ -345,6 +326,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the "Sender" attribute.
      */
+    @Override
     public Address getSender() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -357,6 +339,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     }
 	
 
+    @Override
     public void setSender(Address address) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }    
@@ -364,6 +347,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the desired Recipient type.
      */
+    @Override
     public Address[] getRecipients(Message.RecipientType type)
 				throws MessagingException {
 	checkExpunged();
@@ -381,11 +365,13 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	    return super.getRecipients(type);
     }
 
+    @Override
     public void setRecipients(Message.RecipientType type, Address[] addresses)
 			throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
 
+    @Override
     public void addRecipients(Message.RecipientType type, Address[] addresses)
 			throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
@@ -394,6 +380,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the ReplyTo addresses.
      */
+    @Override
     public Address[] getReplyTo() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -410,6 +397,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return aaclone(envelope.replyTo);
     }
 
+    @Override
     public void setReplyTo(Address[] addresses) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
@@ -417,6 +405,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the decoded subject.
      */
+    @Override
     public String getSubject() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -442,6 +431,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return subject;
     }
 
+    @Override
     public void setSubject(String subject, String charset) 
 		throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
@@ -450,6 +440,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the SentDate.
      */
+    @Override
     public Date getSentDate() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -461,6 +452,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	    return new Date(envelope.date.getTime());
     }
 
+    @Override
     public void setSentDate(Date d) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
@@ -468,6 +460,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the received date (INTERNALDATE).
      */
+    @Override
     public Date getReceivedDate() throws MessagingException {
 	checkExpunged();
 	if (receivedDate == null)
@@ -483,8 +476,30 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
      *
      * Note that this returns RFC822.SIZE.  That is, it's the
      * size of the whole message, header and body included.
+     * Note also that if the size of the message is greater than
+     * Integer.MAX_VALUE (2GB), this method returns Integer.MAX_VALUE.
      */
+    @Override
     public int getSize() throws MessagingException {
+	checkExpunged();
+	// if bodyLoaded, size is already set
+	if (size == -1)
+	    loadEnvelope();	// XXX - could just fetch the size
+	if (size > Integer.MAX_VALUE)
+	    return Integer.MAX_VALUE;	// the best we can do...
+	else
+	    return (int)size;
+    }
+
+    /**
+     * Get the message size as a long. <p>
+     *
+     * Suitable for messages that might be larger than 2GB.
+     * @return	the message size as a long integer
+     * @exception	MessagingException for failures
+     * @since	JavaMail 1.6
+     */
+    public long getSizeLong() throws MessagingException {
 	checkExpunged();
 	// if bodyLoaded, size is already set
 	if (size == -1)
@@ -499,6 +514,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
      * BODYSTRUCTURE. Note that this field is available
      * only for text/plain and message/rfc822 types
      */
+    @Override
     public int getLineCount() throws MessagingException {
 	checkExpunged();
 	// XXX - superclass doesn't implement this
@@ -509,6 +525,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /** 
      * Get the content language.
      */
+    @Override
     public String[] getContentLanguage() throws MessagingException {
     	checkExpunged();
 	if (bodyLoaded)
@@ -520,6 +537,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	    return null;
     }
  
+    @Override
     public void setContentLanguage(String[] languages)
 				throws MessagingException {
     	throw new IllegalWriteException("IMAPMessage is read-only");
@@ -546,6 +564,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
      * Generate this header from the BODYSTRUCTURE. Append parameters
      * as well.
      */
+    @Override
     public synchronized String getContentType() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -564,6 +583,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the Content-Disposition.
      */
+    @Override
     public String getDisposition() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -572,6 +592,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return bs.disposition;
     }
 
+    @Override
     public void setDisposition(String disposition) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
@@ -579,6 +600,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the Content-Transfer-Encoding.
      */
+    @Override
     public String getEncoding() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -590,6 +612,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the Content-ID.
      */
+    @Override
     public String getContentID() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -598,6 +621,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return bs.id;
     }
 
+    @Override
     public void setContentID(String cid) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
@@ -605,6 +629,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the Content-MD5.
      */
+    @Override
     public String getContentMD5() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -613,6 +638,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return bs.md5;
     }
 
+    @Override
     public void setContentMD5(String md5) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
@@ -620,6 +646,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the decoded Content-Description.
      */
+    @Override
     public String getDescription() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -641,6 +668,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return description;
     }
 
+    @Override
     public void setDescription(String description, String charset) 
 			throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
@@ -649,6 +677,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the Message-ID.
      */
+    @Override
     public String getMessageID() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -662,6 +691,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
      * IMAP4rev1). If thats not available, get the "name" ContentType
      * parameter.
      */
+    @Override
     public String getFileName() throws MessagingException {
 	checkExpunged();
 	if (bodyLoaded)
@@ -677,6 +707,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return filename;
     }
 
+    @Override
     public void setFileName(String filename) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
@@ -688,6 +719,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
      *
      * @see javax.mail.internet.MimeMessage#getContentStream
      */
+    @Override
     protected InputStream getContentStream() throws MessagingException {
 	if (bodyLoaded)
 	    return super.getContentStream();
@@ -729,15 +761,21 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	    }
 	}
 
-	if (is == null)
-	    throw new MessagingException("No content");
-	else
-	    return is;
+	if (is == null) {
+	    forceCheckExpunged();	// may throw MessageRemovedException
+	    // nope, the server doesn't think it's expunged.
+	    // can't tell the difference between the server returning NIL
+	    // and some other error that caused null to be returned above,
+	    // so we'll just assume it was empty content.
+	    is = new ByteArrayInputStream(new byte[0]);
+	}
+	return is;
     }
 
     /**
      * Get the DataHandler object for this message.
      */
+    @Override
     public synchronized DataHandler getDataHandler()
 		throws MessagingException {
 	checkExpunged();
@@ -776,6 +814,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return super.getDataHandler();
     }
 
+    @Override
     public void setDataHandler(DataHandler content) 
 			throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
@@ -787,6 +826,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
      * @return	the MIME format stream
      * @since	JavaMail 1.4.5
      */
+    @Override
     public InputStream getMimeStream() throws MessagingException {
 	// XXX - need an "if (bodyLoaded)" version
 	InputStream is = null;
@@ -825,9 +865,11 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 
 	if (is == null) {
 	    forceCheckExpunged();	// may throw MessageRemovedException
-	    // nope, the server doesn't think it's expunged,
-	    // something else is wrong
-	    throw new MessagingException("No content");
+	    // nope, the server doesn't think it's expunged.
+	    // can't tell the difference between the server returning NIL
+	    // and some other error that caused null to be returned above,
+	    // so we'll just assume it was empty content.
+	    is = new ByteArrayInputStream(new byte[0]);
 	}
 	return is;
     }
@@ -835,6 +877,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Write out the bytes into the given OutputStream.
      */
+    @Override
     public void writeTo(OutputStream os)
 				throws IOException, MessagingException {
 	if (bodyLoaded) {
@@ -856,6 +899,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the named header.
      */
+    @Override
     public String[] getHeader(String name) throws MessagingException {
 	checkExpunged();
 
@@ -910,6 +954,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the named header.
      */
+    @Override
     public String getHeader(String name, String delimiter)
 			throws MessagingException {
 	checkExpunged();
@@ -920,16 +965,19 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return headers.getHeader(name, delimiter);
     }
 
+    @Override
     public void setHeader(String name, String value)
 			throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
 
+    @Override
     public void addHeader(String name, String value)
 			throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
 	    
+    @Override
     public void removeHeader(String name)
 			throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
@@ -938,7 +986,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get all headers.
      */
-    @SuppressWarnings("unchecked")
+    @Override
     public Enumeration<Header> getAllHeaders() throws MessagingException {
 	checkExpunged();
 	loadHeaders();
@@ -948,7 +996,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get matching headers.
      */
-    @SuppressWarnings("unchecked")
+    @Override
     public Enumeration<Header> getMatchingHeaders(String[] names)
 			throws MessagingException {
 	checkExpunged();
@@ -959,7 +1007,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get non-matching headers.
      */
-    @SuppressWarnings("unchecked")
+    @Override
     public Enumeration<Header> getNonMatchingHeaders(String[] names)
 			throws MessagingException {
 	checkExpunged();
@@ -967,6 +1015,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	return super.getNonMatchingHeaders(names);
     }
 
+    @Override
     public void addHeaderLine(String line) throws MessagingException {
 	throw new IllegalWriteException("IMAPMessage is read-only");
     }
@@ -974,7 +1023,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get all header-lines.
      */
-    @SuppressWarnings("unchecked")
+    @Override
     public Enumeration<String> getAllHeaderLines() throws MessagingException {
 	checkExpunged();
 	loadHeaders();
@@ -984,7 +1033,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get all matching header-lines.
      */
-    @SuppressWarnings("unchecked")
+    @Override
     public Enumeration<String> getMatchingHeaderLines(String[] names)
 			throws MessagingException {
 	checkExpunged();
@@ -995,7 +1044,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get all non-matching headerlines.
      */
-    @SuppressWarnings("unchecked")
+    @Override
     public Enumeration<String> getNonMatchingHeaderLines(String[] names)
 			throws MessagingException {
 	checkExpunged();
@@ -1006,6 +1055,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Get the Flags for this message.
      */
+    @Override
     public synchronized Flags getFlags() throws MessagingException {
 	checkExpunged();
 	loadFlags();
@@ -1015,6 +1065,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Test if the given Flags are set in this message.
      */
+    @Override
     public synchronized boolean isSet(Flags.Flag flag)
 				throws MessagingException {
 	checkExpunged();
@@ -1025,6 +1076,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
     /**
      * Set/Unset the given flags in this message.
      */
+    @Override
     public synchronized void setFlags(Flags flag, boolean set)
 			throws MessagingException {
         // Acquire MessageCacheLock, to freeze seqnum.
@@ -1108,7 +1160,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	private boolean needMessage = false;
 	private boolean needRDate = false;
 	private String[] hdrs = null;
-	private Set<FetchItem> need = new HashSet<FetchItem>();
+	private Set<FetchItem> need = new HashSet<>();
 
 	/**
 	 * Create a FetchProfileCondition to determine if we need to fetch
@@ -1148,6 +1200,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	 * Return true if we NEED to fetch the requested information
 	 * for the specified message.
 	 */
+	@Override
 	public boolean test(IMAPMessage m) {
 	    if (needEnvelope && m._getEnvelope() == null && !m.bodyLoaded)
 		return true; // no envelope
@@ -1221,7 +1274,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	    // add entry into uid table
 	    if (((IMAPFolder)folder).uidTable == null)
 		((IMAPFolder) folder).uidTable
-			= new Hashtable<Long, IMAPMessage>();
+			= new Hashtable<>();
 	    ((IMAPFolder)folder).uidTable.put(Long.valueOf(u.uid), this);
 	}
 
@@ -1277,7 +1330,6 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 		     * object, because InternetHeaders is not thread
 		     * safe.
 		     */
-		    @SuppressWarnings("unchecked")
 		    Enumeration<Header> e = h.getAllHeaders();
 		    while (e.hasMoreElements()) {
 			Header he = e.nextElement();
@@ -1317,7 +1369,7 @@ public class IMAPMessage extends MimeMessage implements ReadableMime {
 	if (extensionItems == null || extensionItems.isEmpty())
 	    return;
 	if (items == null)
-	    items = new HashMap<String, Object>();
+	    items = new HashMap<>();
 	items.putAll(extensionItems);
     }
 
