@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2019 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2016 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package com.sun.mail.imap;
@@ -44,6 +20,7 @@ import java.lang.reflect.*;
 import java.util.Vector;
 import java.util.StringTokenizer;
 import java.util.Locale;
+import java.util.Properties;
 import java.io.IOException;
 import java.net.InetAddress;
 import java.net.UnknownHostException;
@@ -122,7 +99,7 @@ import java.util.List;
  * The connected IMAPStore object may or may not maintain a separate IMAP
  * protocol object that provides the store a dedicated connection to the
  * IMAP server. This is provided mainly for compatibility with previous
- * implementations of JavaMail and is determined by the value of the 
+ * implementations of Jakarta Mail and is determined by the value of the 
  * mail.imap.separatestoreconnection property. <p>
  *
  * An IMAPStore object provides closed IMAPFolder objects thru its list()
@@ -256,7 +233,7 @@ public class IMAPStore extends Store
 
         // container for the pool's IMAP protocol objects
         private Vector<IMAPProtocol> authenticatedConnections
-		= new Vector<IMAPProtocol>();
+		= new Vector<>();
 
         // vectore of open folders
         private Vector<IMAPFolder> folders;
@@ -336,14 +313,15 @@ public class IMAPStore extends Store
 
 	ConnectionPool(String name, MailLogger plogger, Session session) {
 	    lastTimePruned = System.currentTimeMillis();
+	    Properties props = session.getProperties();
 
-	    boolean debug = PropUtil.getBooleanSessionProperty(session,
+	    boolean debug = PropUtil.getBooleanProperty(props,
 		"mail." + name + ".connectionpool.debug", false);
 	    logger = plogger.getSubLogger("connectionpool",
 					    "DEBUG IMAP CP", debug);
 
 	    // check if the default connection pool size is overridden
-	    int size = PropUtil.getIntSessionProperty(session,
+	    int size = PropUtil.getIntProperty(props,
 		"mail." + name + ".connectionpoolsize", -1);
 	    if (size > 0) {
 		poolSize = size;
@@ -353,7 +331,7 @@ public class IMAPStore extends Store
 		poolSize = 1;
 
 	    // check if the default client-side timeout value is overridden
-	    int connectionPoolTimeout = PropUtil.getIntSessionProperty(session,
+	    int connectionPoolTimeout = PropUtil.getIntProperty(props,
 		"mail." + name + ".connectionpooltimeout", -1);
 	    if (connectionPoolTimeout > 0) {
 		clientTimeoutInterval = connectionPoolTimeout;
@@ -364,7 +342,7 @@ public class IMAPStore extends Store
 		clientTimeoutInterval = 45 * 1000;	// 45 seconds
 
 	    // check if the default server-side timeout value is overridden
-	    int serverTimeout = PropUtil.getIntSessionProperty(session,
+	    int serverTimeout = PropUtil.getIntProperty(props,
 		"mail." + name + ".servertimeout", -1);
 	    if (serverTimeout > 0) {
 		serverTimeoutInterval = serverTimeout;
@@ -375,7 +353,7 @@ public class IMAPStore extends Store
 		serverTimeoutInterval = 30 * 60 * 1000;	// 30 minutes
 
 	    // check if the default server-side timeout value is overridden
-	    int pruning = PropUtil.getIntSessionProperty(session,
+	    int pruning = PropUtil.getIntProperty(props,
 		"mail." + name + ".pruninginterval", -1);
 	    if (pruning > 0) {
 		pruningInterval = pruning;
@@ -388,7 +366,7 @@ public class IMAPStore extends Store
 	    // check to see if we should use a separate (i.e. dedicated)
 	    // store connection
 	    separateStoreConnection =
-		PropUtil.getBooleanSessionProperty(session,
+		PropUtil.getBooleanProperty(props,
 		    "mail." + name + ".separatestoreconnection", false);
 	    if (separateStoreConnection)
 		logger.config("dedicate a store connection");
@@ -406,6 +384,7 @@ public class IMAPStore extends Store
      * that the connection is dead.
      */
     private ResponseHandler nonStoreResponseHandler = new ResponseHandler() {
+	@Override
 	public void handleResponse(Response r) {
 	    // Any of these responses may have a response code.
 	    if (r.isOK() || r.isNO() || r.isBAD() || r.isBYE())
@@ -437,11 +416,13 @@ public class IMAPStore extends Store
     protected IMAPStore(Session session, URLName url,
 				String name, boolean isSSL) {
 	super(session, url); // call super constructor
+	Properties props = session.getProperties();
+
 	if (url != null)
 	    name = url.getProtocol();
 	this.name = name;
 	if (!isSSL)
-	    isSSL = PropUtil.getBooleanSessionProperty(session,
+	    isSSL = PropUtil.getBooleanProperty(props,
 				"mail." + name + ".ssl.enable", false);
 	if (isSSL)
 	    this.defaultPort = 993;
@@ -450,42 +431,43 @@ public class IMAPStore extends Store
 	this.isSSL = isSSL;
 
         debug = session.getDebug();
-	debugusername = PropUtil.getBooleanSessionProperty(session,
+	debugusername = PropUtil.getBooleanProperty(props,
 			"mail.debug.auth.username", true);
-	debugpassword = PropUtil.getBooleanSessionProperty(session,
+	debugpassword = PropUtil.getBooleanProperty(props,
 			"mail.debug.auth.password", false);
 	logger = new MailLogger(this.getClass(),
-			"DEBUG " + name.toUpperCase(Locale.ENGLISH), session);
+			"DEBUG " + name.toUpperCase(Locale.ENGLISH),
+			session.getDebug(), session.getDebugOut());
 
-	boolean partialFetch = PropUtil.getBooleanSessionProperty(session,
+	boolean partialFetch = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".partialfetch", true);
 	if (!partialFetch) {
 	    blksize = -1;
 	    logger.config("mail.imap.partialfetch: false");
 	} else {
-	    blksize = PropUtil.getIntSessionProperty(session,
+	    blksize = PropUtil.getIntProperty(props,
 		"mail." + name +".fetchsize", 1024 * 16);
 	    if (logger.isLoggable(Level.CONFIG))
 		logger.config("mail.imap.fetchsize: " + blksize);
 	}
 
-	ignoreSize = PropUtil.getBooleanSessionProperty(session,
+	ignoreSize = PropUtil.getBooleanProperty(props,
 	    "mail." + name +".ignorebodystructuresize", false);
 	if (logger.isLoggable(Level.CONFIG))
 	    logger.config("mail.imap.ignorebodystructuresize: " + ignoreSize);
 
-	statusCacheTimeout = PropUtil.getIntSessionProperty(session,
+	statusCacheTimeout = PropUtil.getIntProperty(props,
 	    "mail." + name + ".statuscachetimeout", 1000);
 	if (logger.isLoggable(Level.CONFIG))
 	    logger.config("mail.imap.statuscachetimeout: " +
 						statusCacheTimeout);
 
-	appendBufferSize = PropUtil.getIntSessionProperty(session,
+	appendBufferSize = PropUtil.getIntProperty(props,
 	    "mail." + name + ".appendbuffersize", -1);
 	if (logger.isLoggable(Level.CONFIG))
 	    logger.config("mail.imap.appendbuffersize: " + appendBufferSize);
 
-	minIdleTime = PropUtil.getIntSessionProperty(session,
+	minIdleTime = PropUtil.getIntProperty(props,
 	    "mail." + name + ".minidletime", 10);
 	if (logger.isLoggable(Level.CONFIG))
 	    logger.config("mail.imap.minidletime: " + minIdleTime);
@@ -499,19 +481,19 @@ public class IMAPStore extends Store
 	}
 
 	// check if STARTTLS is enabled
-	enableStartTLS = PropUtil.getBooleanSessionProperty(session,
+	enableStartTLS = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".starttls.enable", false);
 	if (enableStartTLS)
 	    logger.config("enable STARTTLS");
 
 	// check if STARTTLS is required
-	requireStartTLS = PropUtil.getBooleanSessionProperty(session,
+	requireStartTLS = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".starttls.required", false);
 	if (requireStartTLS)
 	    logger.config("require STARTTLS");
 
 	// check if SASL is enabled
-	enableSASL = PropUtil.getBooleanSessionProperty(session,
+	enableSASL = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".sasl.enable", false);
 	if (enableSASL)
 	    logger.config("enable SASL");
@@ -522,7 +504,7 @@ public class IMAPStore extends Store
 	    if (s != null && s.length() > 0) {
 		if (logger.isLoggable(Level.CONFIG))
 		    logger.config("SASL mechanisms allowed: " + s);
-		List<String> v = new ArrayList<String>(5);
+		List<String> v = new ArrayList<>(5);
 		StringTokenizer st = new StringTokenizer(s, " ,");
 		while (st.hasMoreTokens()) {
 		    String m = st.nextToken();
@@ -550,25 +532,25 @@ public class IMAPStore extends Store
 	}
 
 	// check if forcePasswordRefresh is enabled
-	forcePasswordRefresh = PropUtil.getBooleanSessionProperty(session,
+	forcePasswordRefresh = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".forcepasswordrefresh", false);
 	if (forcePasswordRefresh)
 	    logger.config("enable forcePasswordRefresh");
 
 	// check if enableimapevents is enabled
-	enableResponseEvents = PropUtil.getBooleanSessionProperty(session,
+	enableResponseEvents = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".enableresponseevents", false);
 	if (enableResponseEvents)
 	    logger.config("enable IMAP response events");
 
 	// check if enableresponseevents is enabled
-	enableImapEvents = PropUtil.getBooleanSessionProperty(session,
+	enableImapEvents = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".enableimapevents", false);
 	if (enableImapEvents)
 	    logger.config("enable IMAP IDLE events");
 
 	// check if message cache debugging set
-	messageCacheDebug = PropUtil.getBooleanSessionProperty(session,
+	messageCacheDebug = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".messagecache.debug", false);
 
 	guid = session.getProperty("mail." + name + ".yahoo.guid");
@@ -576,31 +558,31 @@ public class IMAPStore extends Store
 	    logger.log(Level.CONFIG, "mail.imap.yahoo.guid: {0}", guid);
 
 	// check if throwsearchexception is enabled
-	throwSearchException = PropUtil.getBooleanSessionProperty(session,
+	throwSearchException = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".throwsearchexception", false);
 	if (throwSearchException)
 	    logger.config("throw SearchException");
 
 	// check if peek is set
-	peek = PropUtil.getBooleanSessionProperty(session,
+	peek = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".peek", false);
 	if (peek)
 	    logger.config("peek");
 
 	// check if closeFoldersOnStoreFailure is set
-	closeFoldersOnStoreFailure = PropUtil.getBooleanSessionProperty(session,
+	closeFoldersOnStoreFailure = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".closefoldersonstorefailure", true);
 	if (closeFoldersOnStoreFailure)
 	    logger.config("closeFoldersOnStoreFailure");
 
 	// check if COMPRESS is enabled
-	enableCompress = PropUtil.getBooleanSessionProperty(session,
+	enableCompress = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".compress.enable", false);
 	if (enableCompress)
 	    logger.config("enable COMPRESS");
 
 	// check if finalizeCleanClose is enabled
-	finalizeCleanClose = PropUtil.getBooleanSessionProperty(session,
+	finalizeCleanClose = PropUtil.getBooleanProperty(props,
 	    "mail." + name + ".finalizecleanclose", false);
 	if (finalizeCleanClose)
 	    logger.config("close connection cleanly in finalize");
@@ -649,6 +631,7 @@ public class IMAPStore extends Store
      * being used does not require a password, an empty string or other
      * suitable dummy password should be used.
      */
+    @Override
     protected synchronized boolean 
     protocolConnect(String host, int pport, String user, String password)
 		throws MessagingException {
@@ -669,7 +652,7 @@ public class IMAPStore extends Store
 	if (pport != -1) {
 	    port = pport;
 	} else {
-	    port = PropUtil.getIntSessionProperty(session,
+	    port = PropUtil.getIntProperty(session.getProperties(),
 					"mail." + name + ".port", port);
 	} 
 	
@@ -783,7 +766,7 @@ public class IMAPStore extends Store
 	// issue special ID command to Yahoo! Mail IMAP server
 	// http://en.wikipedia.org/wiki/Yahoo%21_Mail#Free_IMAP_and_SMTPs_access
 	if (guid != null) {
-	    Map<String,String> gmap = new HashMap<String,String>();
+	    Map<String,String> gmap = new HashMap<>();
 	    gmap.put("GUID", guid);
 	    p.id(gmap);
 	}
@@ -839,6 +822,11 @@ public class IMAPStore extends Store
 		p.compress();
 	    }
 	}
+
+	// if server supports UTF-8, enable it for client use
+	// note that this is safe to enable even if mail.mime.allowutf8=false
+	if (p.hasCapability("UTF8=ACCEPT") || p.hasCapability("UTF8=ONLY"))
+	    p.enable("UTF8=ACCEPT");
     }
 
     /**
@@ -881,8 +869,9 @@ public class IMAPStore extends Store
 	    if (mechs == defaultAuthenticationMechanisms) {
 		String dprop = "mail." + name + ".auth." +
 				    m.toLowerCase(Locale.ENGLISH) + ".disable";
-		boolean disabled = PropUtil.getBooleanSessionProperty(
-					session, dprop, m.equals("XOAUTH2"));
+		boolean disabled = PropUtil.getBooleanProperty(
+					session.getProperties(),
+					dprop, m.equals("XOAUTH2"));
 		if (disabled) {
 		    if (logger.isLoggable(Level.FINE))
 			logger.fine("mechanism " + m +
@@ -1056,11 +1045,11 @@ public class IMAPStore extends Store
 			try {
 			    p.removeResponseHandler(nonStoreResponseHandler);
 			    p.disconnect();
-			} finally {
+			} catch (RuntimeException ignored) {
 			    // don't let any exception stop us
-			    p = null;
-			    continue;	// try again, from the top
 			}
+			p = null;
+			continue;   // try again, from the top
 		    }
 		}
 
@@ -1084,11 +1073,11 @@ public class IMAPStore extends Store
 			try {
 			    p.removeResponseHandler(nonStoreResponseHandler);
 			    p.disconnect();
-			} finally {
+			} catch (RuntimeException ignored) {
 			    // don't let any exception stop us
-			    p = null;
-			    continue;	// try again, from the top
 			}
+			p = null;
+			continue;   // try again, from the top
 		    }
 		}
 
@@ -1102,7 +1091,7 @@ public class IMAPStore extends Store
 	    // Add folder to folder-list
 	    if (folder != null) {
                 if (pool.folders == null)
-                    pool.folders = new Vector<IMAPFolder>();
+                    pool.folders = new Vector<>();
 		pool.folders.addElement(folder);
 	    }
         }
@@ -1250,7 +1239,7 @@ public class IMAPStore extends Store
      * do we allow the open to succeed?
      */
     boolean allowReadOnlySelect() {
-	return PropUtil.getBooleanSessionProperty(session,
+	return PropUtil.getBooleanProperty(session.getProperties(),
 	    "mail." + name + ".allowreadonlyselect", false);
     }
 
@@ -1567,6 +1556,7 @@ public class IMAPStore extends Store
      * Check whether this store is connected. Override superclass
      * method, to actually ping our server connection.
      */
+    @Override
     public synchronized boolean isConnected() {
 	if (!super.isConnected()) {
 	    // if we haven't been connected at all, don't bother with
@@ -1607,68 +1597,17 @@ public class IMAPStore extends Store
     /**
      * Close this Store.
      */
+    @Override
     public synchronized void close() throws MessagingException {
-	if (!super.isConnected()) // Already closed.
-	    return;
-
-        IMAPProtocol protocol = null;
-	try {
-	    boolean isEmpty;
-	    synchronized (pool) {
-		// If there's no authenticated connections available
-		// don't create a new one
-		isEmpty = pool.authenticatedConnections.isEmpty();
-	    }
-	    /*
-	     * Have to drop the lock before calling cleanup.
-	     * Yes, there's a potential race here.  The pool could
-	     * become empty after we check, in which case we'll just
-	     * waste time getting a new connection and closing it.
-	     * Or, the pool could be empty now and not empty by the
-	     * time we get into cleanup, but that's ok because cleanup
-	     * will just close the connection.
-	     */
-	    if (isEmpty) {
-		pool.logger.fine("close() - no connections ");
-		cleanup();
-		return;
-	    }
-
-            protocol = getStoreProtocol();
-	    /*
-	     * We have to remove the protocol from the pool so that,
-	     * when our response handler processes the BYE response
-	     * and calls cleanup, which calls emptyConnection, that
-	     * we don't try to log out this connection twice.
-	     */
-	    synchronized (pool) {
-                pool.authenticatedConnections.removeElement(protocol);
-	    }
-
-	    /*
-	     * LOGOUT. 
-	     *
-	     * Note that protocol.logout() closes the server socket
-	     * connection, regardless of what happens ..
-	     *
-	     * Also note that protocol.logout() results in a BYE
-	     * response (As per RFC 3501, BYE is a *required* response
-	     * to LOGOUT). In fact, even if protocol.logout() fails
-	     * with an IOException (if the server connection is dead),
-	     * iap.Protocol.command() converts that exception into a 
-	     * BYE response. So, I depend on my BYE handler to set the
-	     * flag that causes releaseStoreProtocol to do the
-	     * Store cleanup.
-	     */
-	    protocol.logout();
-	} catch (ProtocolException pex) { 
-	    // Hmm .. will this ever happen ?
-	    throw new MessagingException(pex.getMessage(), pex);
-        } finally {
-            releaseStoreProtocol(protocol);
-        }
+	cleanup();
+	// do these again in case cleanup returned early
+	// because we were already closed due to a failure,
+	// in which case we force close everything
+	closeAllFolders(true);
+	emptyConnectionPool(true);
     }
 
+    @Override
     protected void finalize() throws Throwable {
 	if (!finalizeCleanClose) {
 	    // when finalizing, close connections abruptly
@@ -1711,6 +1650,24 @@ public class IMAPStore extends Store
 	    logger.fine("IMAPStore cleanup, force " + force);
 
 	if (!force || closeFoldersOnStoreFailure) {
+	    closeAllFolders(force);
+	}
+
+	emptyConnectionPool(force);
+
+	// to set the state and send the closed connection event
+	try {
+	    super.close();
+	} catch (MessagingException mex) {
+	    // ignore it
+	}
+	logger.fine("IMAPStore cleanup done");
+    }
+
+    /**
+     * Close all open Folders.  If force is true, close them forcibly.
+     */
+    private void closeAllFolders(boolean force) {
         List<IMAPFolder> foldersCopy = null;
         boolean done = true;
 
@@ -1759,25 +1716,13 @@ public class IMAPStore extends Store
 	    }
 
 	}
-	}
-
-        synchronized (pool) {
-	    emptyConnectionPool(force);
-	}
-
-	// to set the state and send the closed connection event
-	try {
-	    super.close();
-	} catch (MessagingException mex) {
-	    // ignore it
-	}
-	logger.fine("IMAPStore cleanup done");
     }
 
     /**
      * Get the default folder, representing the root of this user's 
      * namespace. Returns a closed DefaultFolder object.
      */
+    @Override
     public synchronized Folder getDefaultFolder() throws MessagingException {
 	checkConnected();
 	return new DefaultFolder(this);
@@ -1786,6 +1731,7 @@ public class IMAPStore extends Store
     /**
      * Get named folder. Returns a new, closed IMAPFolder.
      */
+    @Override
     public synchronized Folder getFolder(String name)
 				throws MessagingException {
 	checkConnected();
@@ -1795,6 +1741,7 @@ public class IMAPStore extends Store
     /**
      * Get named folder. Returns a new, closed IMAPFolder.
      */
+    @Override
     public synchronized Folder getFolder(URLName url)
 				throws MessagingException {
 	checkConnected();
@@ -1867,6 +1814,7 @@ public class IMAPStore extends Store
      * Using the IMAP NAMESPACE command (RFC 2342), return a set
      * of folders representing the Personal namespaces.
      */
+    @Override
     public Folder[] getPersonalNamespaces() throws MessagingException {
 	Namespaces ns = getNamespaces();
 	if (ns == null || ns.personal == null)
@@ -1878,6 +1826,7 @@ public class IMAPStore extends Store
      * Using the IMAP NAMESPACE command (RFC 2342), return a set
      * of folders representing the User's namespaces.
      */
+    @Override
     public Folder[] getUserNamespaces(String user)
 				throws MessagingException {
 	Namespaces ns = getNamespaces();
@@ -1890,6 +1839,7 @@ public class IMAPStore extends Store
      * Using the IMAP NAMESPACE command (RFC 2342), return a set
      * of folders representing the Shared namespaces.
      */
+    @Override
     public Folder[] getSharedNamespaces() throws MessagingException {
 	Namespaces ns = getNamespaces();
 	if (ns == null || ns.shared == null)
@@ -1955,6 +1905,7 @@ public class IMAPStore extends Store
      * @exception MessagingException	if the server doesn't support the
      *					QUOTA extension
      */
+    @Override
     public synchronized Quota[] getQuota(String root)
 				throws MessagingException {
 	checkConnected();
@@ -1985,6 +1936,7 @@ public class IMAPStore extends Store
      * @exception MessagingException	if the server doesn't support the
      *					QUOTA extension
      */
+    @Override
     public synchronized void setQuota(Quota quota) throws MessagingException {
 	checkConnected();
         IMAPProtocol p = null;
@@ -2011,6 +1963,7 @@ public class IMAPStore extends Store
     /**
      * Response handler method.
      */
+    @Override
     public void handleResponse(Response r) {
 	// Any of these responses may have a response code.
 	if (r.isOK() || r.isNO() || r.isBAD() || r.isBYE())
