@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2018 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2016 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package com.sun.mail.pop3;
@@ -85,18 +61,21 @@ public class POP3Folder extends Folder {
 	this.store = store;
 	if (name.equalsIgnoreCase("INBOX"))
 	    exists = true;
-	logger = new MailLogger(this.getClass(),
-				"DEBUG POP3", store.getSession());
+	logger = new MailLogger(this.getClass(), "DEBUG POP3",
+	    store.getSession().getDebug(), store.getSession().getDebugOut());
     }
 
+    @Override
     public String getName() {
 	return name;
     }
 
+    @Override
     public String getFullName() {
 	return name;
     }
 
+    @Override
     public Folder getParent() {
 	return new DefaultFolder(store);
     }
@@ -107,6 +86,7 @@ public class POP3Folder extends Folder {
      *
      * @return	true for INBOX, false otherwise
      */
+    @Override
     public boolean exists() {
 	return exists;
     }
@@ -117,6 +97,7 @@ public class POP3Folder extends Folder {
      *
      * @exception	MessagingException	always
      */
+    @Override
     public Folder[] list(String pattern) throws MessagingException {
 	throw new MessagingException("not a directory");
     }
@@ -126,6 +107,7 @@ public class POP3Folder extends Folder {
      *
      * @return	NUL
      */
+    @Override
     public char getSeparator() {
 	return '\0';
     }
@@ -135,6 +117,7 @@ public class POP3Folder extends Folder {
      *
      * @return	Folder.HOLDS_MESSAGES
      */
+    @Override
     public int getType() {
 	return HOLDS_MESSAGES;
     }
@@ -145,6 +128,7 @@ public class POP3Folder extends Folder {
      *
      * @return	false
      */
+    @Override
     public boolean create(int type) throws MessagingException {
 	return false;
     }
@@ -155,6 +139,7 @@ public class POP3Folder extends Folder {
      *
      * @return	false
      */
+    @Override
     public boolean hasNewMessages() throws MessagingException {
 	return false;    // no way to know
     }
@@ -165,6 +150,7 @@ public class POP3Folder extends Folder {
      *
      * @exception	MessagingException	always
      */
+    @Override
     public Folder getFolder(String name) throws MessagingException {
 	throw new MessagingException("not a directory");
     }
@@ -176,6 +162,7 @@ public class POP3Folder extends Folder {
      *
      * @exception	MethodNotSupportedException	always
      */
+    @Override
     public boolean delete(boolean recurse) throws MessagingException {
 	throw new MethodNotSupportedException("delete");
     }
@@ -186,6 +173,7 @@ public class POP3Folder extends Folder {
      *
      * @exception	MethodNotSupportedException	always
      */
+    @Override
     public boolean renameTo(Folder f) throws MessagingException {
 	throw new MethodNotSupportedException("renameTo");
     }
@@ -198,6 +186,7 @@ public class POP3Folder extends Folder {
      * @exception	AuthenticationFailedException	authentication failures
      * @exception	MessagingException	other open failures
      */
+    @Override
     public synchronized void open(int mode) throws MessagingException {
 	checkClosed();
 	if (!exists)
@@ -238,6 +227,7 @@ public class POP3Folder extends Folder {
 	notifyConnectionListeners(ConnectionEvent.OPENED);
     }
 
+    @Override
     public synchronized void close(boolean expunge) throws MessagingException {
 	checkOpen();
 
@@ -297,6 +287,7 @@ public class POP3Folder extends Folder {
 	}
     }
 
+    @Override
     public synchronized boolean isOpen() {
 	if (!opened)
 	    return false;
@@ -308,9 +299,8 @@ public class POP3Folder extends Folder {
 		close(false);
 	    } catch (MessagingException mex) {
 		// ignore it
-	    } finally {
-		return false;
 	    }
+	    return false;
 	}
 	return true;
     }
@@ -321,6 +311,7 @@ public class POP3Folder extends Folder {
      *
      * @return	empty Flags object
      */
+    @Override
     public Flags getPermanentFlags() {
 	return new Flags(); // empty flags object
     }
@@ -330,6 +321,7 @@ public class POP3Folder extends Folder {
      * protocol doesn't support notification of new messages
      * arriving in open folders.
      */
+    @Override
     public synchronized int getMessageCount() throws MessagingException {
 	if (!opened)
 	    return -1;
@@ -337,6 +329,7 @@ public class POP3Folder extends Folder {
 	return total;
     }
 
+    @Override
     public synchronized Message getMessage(int msgno)
 					throws MessagingException {
 	checkOpen();
@@ -374,6 +367,7 @@ public class POP3Folder extends Folder {
      *
      * @exception	MethodNotSupportedException	always
      */
+    @Override
     public void appendMessages(Message[] msgs) throws MessagingException {
 	throw new MethodNotSupportedException("Append not supported");	
     }
@@ -387,6 +381,7 @@ public class POP3Folder extends Folder {
      *
      * @exception	MethodNotSupportedException	always
      */
+    @Override
     public Message[] expunge() throws MessagingException {
 	throw new MethodNotSupportedException("Expunge not supported");
     }
@@ -400,6 +395,7 @@ public class POP3Folder extends Folder {
      * the headers and size of all messages are fetched using the POP3 TOP
      * and LIST commands.
      */
+    @Override
     public synchronized void fetch(Message[] msgs, FetchProfile fp)
 				throws MessagingException {
 	checkReadable();
@@ -548,6 +544,7 @@ public class POP3Folder extends Folder {
     /**
      * Close the folder when we're finalized.
      */
+    @Override
     protected void finalize() throws Throwable {
 	forceClose = !store.finalizeCleanClose;
 	try {
@@ -600,6 +597,7 @@ public class POP3Folder extends Folder {
     /*
      * Only here to make accessible to POP3Message.
      */
+    @Override
     protected void notifyMessageChangedListeners(int type, Message m) {
 	super.notifyMessageChangedListeners(type, m);
     }
