@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2019 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2013 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package javax.mail;
@@ -64,7 +40,7 @@ import java.util.*;
  *
  * <strong>Warning:</strong>
  * Serialized objects of this class may not be compatible with future
- * JavaMail API releases.  The current serialization support is
+ * Jakarta Mail API releases.  The current serialization support is
  * appropriate for short term storage. <p>
  *
  * The below code sample illustrates how to set, examine, and get the 
@@ -100,6 +76,8 @@ import java.util.*;
 public class Flags implements Cloneable, Serializable {
 
     private int system_flags = 0;
+    // used as a case-independent Set that preserves the original case,
+    // the key is the lowercase flag name and the value is the original
     private Hashtable<String, String> user_flags = null;
 
     private final static int ANSWERED_BIT 	= 0x01;
@@ -153,7 +131,7 @@ public class Flags implements Cloneable, Serializable {
 
 	/**
 	 * This message is seen. This flag is implicitly set by the 
-	 * implementation when the this Message's content is returned 
+	 * implementation when this Message's content is returned 
 	 * to the client in some form. The <code>getInputStream</code>
 	 * and <code>getContent</code> methods on Message cause this
 	 * flag to be set. <p>
@@ -213,7 +191,7 @@ public class Flags implements Cloneable, Serializable {
      * @param flag	the flag for initialization
      */
     public Flags(String flag) {
-	user_flags = new Hashtable<String, String>(1);
+	user_flags = new Hashtable<>(1);
 	user_flags.put(flag.toLowerCase(Locale.ENGLISH), flag);
     }
 
@@ -233,7 +211,7 @@ public class Flags implements Cloneable, Serializable {
      */
     public void add(String flag) {
 	if (user_flags == null)
-	    user_flags = new Hashtable<String, String>(1);
+	    user_flags = new Hashtable<>(1);
 	user_flags.put(flag.toLowerCase(Locale.ENGLISH), flag);
     }
 
@@ -248,7 +226,7 @@ public class Flags implements Cloneable, Serializable {
 
 	if (f.user_flags != null) { // add user-defined flags
 	    if (user_flags == null)
-		user_flags = new Hashtable<String, String>(1);
+		user_flags = new Hashtable<>(1);
 
 	    Enumeration<String> e = f.user_flags.keys();
 
@@ -297,6 +275,45 @@ public class Flags implements Cloneable, Serializable {
 	}
     }
 
+    /**
+     * Remove any flags <strong>not</strong> in the given Flags object.
+     * Useful for clearing flags not supported by a server.  If the
+     * given Flags object includes the Flags.Flag.USER flag, all user
+     * flags in this Flags object are retained.
+     *
+     * @param	f	the flags to keep
+     * @return		true if this Flags object changed
+     * @since		JavaMail 1.6
+     */
+    public boolean retainAll(Flags f) {
+	boolean changed = false;
+	int sf = system_flags & f.system_flags;
+	if (system_flags != sf) {
+	    system_flags = sf;
+	    changed = true;
+	}
+
+	// if we have user flags, and the USER flag is not set in "f",
+	// determine which user flags to clear
+	if (user_flags != null && (f.system_flags & USER_BIT) == 0) {
+	    if (f.user_flags != null) {
+		Enumeration<String> e = user_flags.keys();
+		while (e.hasMoreElements()) {
+		    String key = e.nextElement();
+		    if (!f.user_flags.containsKey(key)) {
+			user_flags.remove(key);
+			changed = true;
+		    }
+		}
+	    } else {
+		// if anything in user_flags, throw them away
+		changed = user_flags.size() > 0;
+		user_flags = null;
+	    }
+	}
+	return changed;
+    }
+
     /**
      * Check whether the specified system flag is present in this Flags object.
      *
@@ -354,6 +371,7 @@ public class Flags implements Cloneable, Serializable {
      *
      * @return	true if they're equal
      */
+    @Override
     public boolean equals(Object obj) {
 	if (!(obj instanceof Flags))
 	    return false;
@@ -365,18 +383,12 @@ public class Flags implements Cloneable, Serializable {
 	    return false;
 
 	// Check user flags
-	if (f.user_flags == null && this.user_flags == null)
+	int size = this.user_flags == null ? 0 : this.user_flags.size();
+	int fsize = f.user_flags == null ? 0 : f.user_flags.size();
+	if (size == 0 && fsize == 0)
 	    return true;
-	if (f.user_flags != null && this.user_flags != null &&
-		f.user_flags.size() == this.user_flags.size()) {
-	    Enumeration<String> e = f.user_flags.keys();
-
-	    while (e.hasMoreElements()) {
-		if (!this.user_flags.containsKey(e.nextElement()))
-		    return false;
-	    }
-	    return true;
-	}
+	if (f.user_flags != null && this.user_flags != null && fsize == size)
+	    return user_flags.keySet().equals(f.user_flags.keySet());
 
 	return false;
     }
@@ -386,6 +398,7 @@ public class Flags implements Cloneable, Serializable {
      *
      * @return	the hash code
      */
+    @Override
     public int hashCode() {
 	int hash = system_flags;
 	if (user_flags != null) {
@@ -403,7 +416,7 @@ public class Flags implements Cloneable, Serializable {
      * @return	array of Flags.Flag objects representing system flags
      */
     public Flag[] getSystemFlags() {
-	Vector<Flag> v = new Vector<Flag>();
+	Vector<Flag> v = new Vector<>();
 	if ((system_flags & ANSWERED_BIT) != 0)
 	    v.addElement(Flag.ANSWERED);
 	if ((system_flags & DELETED_BIT) != 0)
@@ -431,7 +444,7 @@ public class Flags implements Cloneable, Serializable {
      * @return	array of Strings, each String represents a flag.
      */
     public String[] getUserFlags() {
-	Vector<String> v = new Vector<String>();
+	Vector<String> v = new Vector<>();
 	if (user_flags != null) {
 	    Enumeration<String> e = user_flags.elements();
 
@@ -444,10 +457,29 @@ public class Flags implements Cloneable, Serializable {
 	return f;
     }
 
+    /**
+     * Clear all of the system flags.
+     *
+     * @since	JavaMail 1.6
+     */
+    public void clearSystemFlags() {
+	system_flags = 0;
+    }
+
+    /**
+     * Clear all of the user flags.
+     *
+     * @since	JavaMail 1.6
+     */
+    public void clearUserFlags() {
+	user_flags = null;
+    }
+
     /**
      * Returns a clone of this Flags object.
      */
     @SuppressWarnings("unchecked")
+    @Override
     public Object clone() {
 	Flags f = null;
 	try {
@@ -460,6 +492,47 @@ public class Flags implements Cloneable, Serializable {
 	return f;
     }
 
+    /**
+     * Return a string representation of this Flags object.
+     * Note that the exact format of the string is subject to change.
+     */
+    public String toString() {
+	StringBuilder sb = new StringBuilder();
+
+	if ((system_flags & ANSWERED_BIT) != 0)
+	    sb.append("\\Answered ");
+	if ((system_flags & DELETED_BIT) != 0)
+	    sb.append("\\Deleted ");
+	if ((system_flags & DRAFT_BIT) != 0)
+	    sb.append("\\Draft ");
+	if ((system_flags & FLAGGED_BIT) != 0)
+	    sb.append("\\Flagged ");
+	if ((system_flags & RECENT_BIT) != 0)
+	    sb.append("\\Recent ");
+	if ((system_flags & SEEN_BIT) != 0)
+	    sb.append("\\Seen ");
+	if ((system_flags & USER_BIT) != 0)
+	    sb.append("\\* ");
+
+	boolean first = true;
+	if (user_flags != null) {
+	    Enumeration<String> e = user_flags.elements();
+
+	    while (e.hasMoreElements()) {
+		if (first)
+		    first = false;
+		else
+		    sb.append(' ');
+		sb.append(e.nextElement());
+	    }
+	}
+
+	if (first && sb.length() > 0)
+	    sb.setLength(sb.length() - 1);	// smash trailing space
+
+	return sb.toString();
+    }
+
     /*****
     public static void main(String argv[]) throws Exception {
 	// a new flags object
