@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2018 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2013 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package javax.mail;
@@ -214,12 +190,12 @@ public abstract class Transport extends Service {
 	 * indexed by address type
 	 */
 	Map<String, List<Address>> protocols
-		= new HashMap<String, List<Address>>();
+		= new HashMap<>();
 
 	// Lists of addresses
-	List<Address> invalid = new ArrayList<Address>();
-	List<Address> validSent = new ArrayList<Address>();
-	List<Address> validUnsent = new ArrayList<Address>();
+	List<Address> invalid = new ArrayList<>();
+	List<Address> validSent = new ArrayList<>();
+	List<Address> validUnsent = new ArrayList<>();
 
 	for (int i = 0; i < addresses.length; i++) {
 	    // is this address type already in the map?
@@ -228,7 +204,7 @@ public abstract class Transport extends Service {
 		v.add(addresses[i]);
 	    } else {
 		// need to add a new protocol
-		List<Address> w = new ArrayList<Address>();
+		List<Address> w = new ArrayList<>();
 		w.add(addresses[i]);
 		protocols.put(addresses[i].getType(), w);
 	    }
@@ -377,7 +353,7 @@ public abstract class Transport extends Service {
      */
     public synchronized void addTransportListener(TransportListener l) {
 	if (transportListeners == null)
-	    transportListeners = new Vector<TransportListener>();
+	    transportListeners = new Vector<>();
 	transportListeners.addElement(l);
     }
 
