@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2019 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2014 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package javax.mail;
@@ -189,10 +165,11 @@ public class URLName {
     /**
      * Constructs a string representation of this URLName.
      */
+    @Override
     public String toString() {
 	if (fullURL == null) {
 	    // add the "protocol:"
-	    StringBuffer tempURL = new StringBuffer();
+	    StringBuilder tempURL = new StringBuilder();
 	    if (protocol != null) {
 		tempURL.append(protocol);
 		tempURL.append(":");
@@ -409,7 +386,13 @@ public class URLName {
      * @exception	MalformedURLException if the URL is malformed
      */
     public URL getURL() throws MalformedURLException {
-        return new URL(getProtocol(), getHost(), getPort(), getFile());
+	// URL expects the file to include the separating "/"
+	String f = getFile();
+	if (f == null)
+	    f = "";
+	else
+	    f = "/" + f;
+        return new URL(getProtocol(), getHost(), getPort(), f);
     }
 
     /**
@@ -433,6 +416,7 @@ public class URLName {
      * Note also that the password field is not included in the comparison,
      * nor is any reference field appended to the filename.
      */
+    @Override
     public boolean equals(Object obj) {
         if (!(obj instanceof URLName))
 	    return false;
@@ -485,6 +469,7 @@ public class URLName {
     /**
      * Compute the hash code for this URLName.
      */
+    @Override
     public int hashCode() {
 	if (hashCode != 0)
 	    return hashCode;
@@ -589,7 +574,7 @@ public class URLName {
 
     private static String _encode(String s) {
 	int maxBytesPerChar = 10;
-        StringBuffer out = new StringBuffer(s.length());
+        StringBuilder out = new StringBuilder(s.length());
 	ByteArrayOutputStream buf = new ByteArrayOutputStream(maxBytesPerChar);
 	OutputStreamWriter writer = new OutputStreamWriter(buf);
 
@@ -658,7 +643,7 @@ public class URLName {
 
     /**
      * Decodes a &quot;x-www-form-urlencoded&quot; 
-     * to a <tt>String</tt>.
+     * to a <code>String</code>.
      * @param s the <code>String</code> to decode
      * @return the newly decoded <code>String</code>
      */
@@ -668,7 +653,7 @@ public class URLName {
 	if (indexOfAny(s, "+%") == -1)
 	    return s;		// the common case
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         for (int i = 0; i < s.length(); i++) {
             char c = s.charAt(i);
             switch (c) {
