@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2019 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2017 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://oss.oracle.com/licenses/CDDL+GPL-1.1
- * or LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package javax.mail.internet;
@@ -64,11 +40,11 @@ import com.sun.mail.util.PropUtil;
  * content. The intent is that service providers, mail JavaBean writers
  * and mail clients will write many such subclasses and their Command
  * Beans, and will install them into the JavaBeans Activation
- * Framework, so that any JavaMail implementation and its clients can
+ * Framework, so that any Jakarta Mail implementation and its clients can
  * transparently find and use these classes. Thus, a MIME multipart
  * handler is treated just like any other type handler, thereby
  * decoupling the process of providing multipart handlers from the
- * JavaMail API. Lacking these additional MimeMultipart subclasses,
+ * Jakarta Mail API. Lacking these additional MimeMultipart subclasses,
  * all subtypes of MIME multipart data appear as MimeMultipart objects. <p>
  *
  * An application can directly construct a MIME multipart object of any
@@ -618,7 +594,6 @@ public class MimeMultipart extends Multipart {
 	    LineInputStream lin = new LineInputStream(in);
 	    StringBuilder preamblesb = null;
 	    String line;
-	    String lineSeparator = null;
 	    while ((line = lin.readLine()) != null) {
 		/*
 		 * Strip trailing whitespace.  Can't use trim method
@@ -664,20 +639,10 @@ public class MimeMultipart extends Multipart {
 
 		// save the preamble after skipping blank lines
 		if (line.length() > 0) {
-		    // if we haven't figured out what the line separator
-		    // is, do it now
-		    if (lineSeparator == null) {
-			try {
-			    lineSeparator =
-				System.getProperty("line.separator", "\n");
-			} catch (SecurityException ex) {
-			    lineSeparator = "\n";
-			}
-		    }
 		    // accumulate the preamble
 		    if (preamblesb == null)
 			preamblesb = new StringBuilder(line.length() + 2);
-		    preamblesb.append(line).append(lineSeparator);
+		    preamblesb.append(line).append(System.lineSeparator());
 		}
 	    }
 
