@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2019 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2014 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package javax.mail.internet;
@@ -45,7 +21,21 @@ import javax.mail.EncodingAware;
 import javax.activation.*;
 import java.util.*;
 import java.io.*;
-import com.sun.mail.util.*;
+import java.nio.charset.Charset;
+import com.sun.mail.util.PropUtil;
+import com.sun.mail.util.ASCIIUtility;
+import com.sun.mail.util.BASE64DecoderStream;
+import com.sun.mail.util.BASE64EncoderStream;
+import com.sun.mail.util.BEncoderStream;
+import com.sun.mail.util.LineInputStream;
+import com.sun.mail.util.LineOutputStream;
+import com.sun.mail.util.LogOutputStream;
+import com.sun.mail.util.QDecoderStream;
+import com.sun.mail.util.QEncoderStream;
+import com.sun.mail.util.QPDecoderStream;
+import com.sun.mail.util.QPEncoderStream;
+import com.sun.mail.util.UUDecoderStream;
+import com.sun.mail.util.UUEncoderStream;
 
 /**
  * This is a utility class that provides various MIME related
@@ -54,7 +44,7 @@ import com.sun.mail.util.*;
  * There are a set of methods to encode and decode MIME headers as 
  * per RFC 2047.  Note that, in general, these methods are
  * <strong>not</strong> needed when using methods such as
- * <code>setSubject</code> and <code>setRecipients</code>; JavaMail
+ * <code>setSubject</code> and <code>setRecipients</code>; Jakarta Mail
  * will automatically encode and decode data when using these "higher
  * level" methods.  The methods below are only needed when maniuplating
  * raw MIME headers using <code>setHeader</code> and <code>getHeader</code>
@@ -149,7 +139,7 @@ public class MimeUtility {
     // cached map of whether a charset is compatible with ASCII
     // Map<String,Boolean>
     private static final Map<String, Boolean> nonAsciiCharsetMap
-	    = new HashMap<String, Boolean>();
+	    = new HashMap<>();
 
     private static final boolean decodeStrict =
 	PropUtil.getBooleanSystemProperty("mail.mime.decodetext.strict", true);
@@ -158,6 +148,8 @@ public class MimeUtility {
     private static final boolean ignoreUnknownEncoding =
 	PropUtil.getBooleanSystemProperty(
 	    "mail.mime.ignoreunknownencoding", false);
+    private static final boolean allowUtf8 =
+	PropUtil.getBooleanSystemProperty("mail.mime.allowutf8", false);
     /*
      * The following two properties allow disabling the fold()
      * and unfold() methods and reverting to the previous behavior.
@@ -593,8 +585,8 @@ public class MimeUtility {
 	// Encoded words found. Start decoding ...
 
 	st = new StringTokenizer(etext, lwsp, true);
-	StringBuffer sb = new StringBuffer();  // decode buffer
-	StringBuffer wsb = new StringBuffer(); // white space buffer
+	StringBuilder sb = new StringBuilder();  // decode buffer
+	StringBuilder wsb = new StringBuilder(); // white space buffer
 	boolean prevWasEncoded = false;
 
 	while (st.hasMoreTokens()) {
@@ -754,7 +746,7 @@ public class MimeUtility {
 	    throw new UnsupportedEncodingException(
 			"Unknown transfer encoding: " + encoding);
 
-	StringBuffer outb = new StringBuffer(); // the output buffer
+	StringBuilder outb = new StringBuilder(); // the output buffer
 	doEncode(string, b64, jcharset, 
 		 // As per RFC 2047, size of an encoded string should not
 		 // exceed 75 bytes.
@@ -768,7 +760,7 @@ public class MimeUtility {
 
     private static void doEncode(String string, boolean b64, 
 		String jcharset, int avail, String prefix, 
-		boolean first, boolean encodingWord, StringBuffer buf) 
+		boolean first, boolean encodingWord, StringBuilder buf)
 			throws UnsupportedEncodingException {
 
 	// First find out what the length of the encoded version of
@@ -809,7 +801,7 @@ public class MimeUtility {
 
 	    byte[] encodedBytes = os.toByteArray(); // the encoded stuff
 	    // Now write out the encoded (all ASCII) bytes into our
-	    // StringBuffer
+	    // StringBuilder
 	    if (!first) // not the first line of this sequence
 		if (foldEncodedWords)
 		    buf.append("\r\n "); // start a continuation line
@@ -946,7 +938,7 @@ public class MimeUtility {
     private static String decodeInnerWords(String word)
 				throws UnsupportedEncodingException {
 	int start = 0, i;
-	StringBuffer buf = new StringBuffer();
+	StringBuilder buf = new StringBuilder();
 	while ((i = word.indexOf("=?", start)) >= 0) {
 	    buf.append(word.substring(start, i));
 	    // find first '?' after opening '=?' - end of charset
@@ -1007,7 +999,7 @@ public class MimeUtility {
 	    char c = word.charAt(i);
 	    if (c == '"' || c == '\\' || c == '\r' || c == '\n') {
 		// need to escape them and then quote the whole string
-		StringBuffer sb = new StringBuffer(len + 3);
+		StringBuilder sb = new StringBuilder(len + 3);
 		sb.append('"');
 		sb.append(word.substring(0, i));
 		int lastc = 0;
@@ -1024,13 +1016,14 @@ public class MimeUtility {
 		}
 		sb.append('"');
 		return sb.toString();
-	    } else if (c < 040 || c >= 0177 || specials.indexOf(c) >= 0)
+	    } else if (c < 040 || (c >= 0177 && !allowUtf8) ||
+		    specials.indexOf(c) >= 0)
 		// These characters cause the string to be quoted
 		needQuoting = true;
 	}
 
 	if (needQuoting) {
-	    StringBuffer sb = new StringBuffer(len + 2);
+	    StringBuilder sb = new StringBuilder(len + 2);
 	    sb.append('"').append(word).append('"');
 	    return sb.toString();
 	} else 
@@ -1156,44 +1149,39 @@ public class MimeUtility {
 	if (!foldText)
 	    return s;
 
-	StringBuffer sb = null;
+	StringBuilder sb = null;
 	int i;
 	while ((i = indexOfAny(s, "\r\n")) >= 0) {
 	    int start = i;
-	    int l = s.length();
+	    int slen = s.length();
 	    i++;		// skip CR or NL
-	    if (i < l && s.charAt(i - 1) == '\r' && s.charAt(i) == '\n')
+	    if (i < slen && s.charAt(i - 1) == '\r' && s.charAt(i) == '\n')
 		i++;	// skip LF
-	    if (start == 0 || s.charAt(start - 1) != '\\') {
-		char c;
-		// if next line starts with whitespace, skip all of it
-		// XXX - always has to be true?
-		if (i < l && ((c = s.charAt(i)) == ' ' || c == '\t')) {
-		    i++;	// skip whitespace
-		    while (i < l && ((c = s.charAt(i)) == ' ' || c == '\t'))
-			i++;
-		    if (sb == null)
-			sb = new StringBuffer(s.length());
-		    if (start != 0) {
-			sb.append(s.substring(0, start));
-			sb.append(' ');
-		    }
-		    s = s.substring(i);
-		    continue;
-		}
-		// it's not a continuation line, just leave it in
-		if (sb == null)
-		    sb = new StringBuffer(s.length());
-		sb.append(s.substring(0, i));
-		s = s.substring(i);
-	    } else {
-		// there's a backslash at "start - 1"
+	    if (start > 0 && s.charAt(start - 1) == '\\') {
+		// there's a backslash before the line break
 		// strip it out, but leave in the line break
 		if (sb == null)
-		    sb = new StringBuffer(s.length());
+		    sb = new StringBuilder(s.length());
 		sb.append(s.substring(0, start - 1));
 		sb.append(s.substring(start, i));
 		s = s.substring(i);
+	    } else {
+		char c;
+		// if next line starts with whitespace,
+		// or at the end of the string, remove the line break
+		// XXX - next line should always start with whitespace
+		if (i >= slen || (c = s.charAt(i)) == ' ' || c == '\t') {
+		    if (sb == null)
+			sb = new StringBuilder(s.length());
+		    sb.append(s.substring(0, start));
+		    s = s.substring(i);
+		} else {
+		    // it's not a continuation line, just leave in the newline
+		    if (sb == null)
+			sb = new StringBuilder(s.length());
+		    sb.append(s.substring(0, i));
+		    s = s.substring(i);
+		}
 	    }
 	}
 	if (sb != null) {
@@ -1239,6 +1227,14 @@ public class MimeUtility {
 	    return charset;
 
 	String alias = mime2java.get(charset.toLowerCase(Locale.ENGLISH));
+	if (alias != null) {
+	    // verify that the mapped name is valid before trying to use it
+	    try {
+		Charset.forName(alias);
+	    } catch (Exception ex) {
+		alias = null;	// charset alias not valid, use original name
+	    }
+	}
 	return alias == null ? charset : alias;
     }
 
@@ -1298,6 +1294,7 @@ public class MimeUtility {
 	    } catch (SecurityException sex) {
 		
 		class NullInputStream extends InputStream {
+		    @Override
 		    public int read() {
 			return 0;
 		    }
@@ -1333,8 +1330,8 @@ public class MimeUtility {
     private static Map<String, String> java2mime;
 
     static {
-	java2mime = new HashMap<String, String>(40);
-	mime2java = new HashMap<String, String>(10);
+	java2mime = new HashMap<>(40);
+	mime2java = new HashMap<>(14);
 
 	try {
 	    // Use this class's classloader to load the mapping file
@@ -1422,6 +1419,10 @@ public class MimeUtility {
 	    mime2java.put("euckr", "KSC5601");
 	    mime2java.put("us-ascii", "ISO-8859-1");
 	    mime2java.put("x-us-ascii", "ISO-8859-1");
+	    mime2java.put("gb2312", "GB18030");
+	    mime2java.put("cp936", "GB18030");
+	    mime2java.put("ms936", "GB18030");
+	    mime2java.put("gbk", "GB18030");
 	}
     }
 
@@ -1641,14 +1642,17 @@ class AsciiOutputStream extends OutputStream {
 	checkEOL = encodeEolStrict && breakOnNonAscii;
     }
 
+    @Override
     public void write(int b) throws IOException {
 	check(b);
     }
 
+    @Override
     public void write(byte b[]) throws IOException {
 	write(b, 0, b.length);
     }
 
+    @Override
     public void write(byte b[], int off, int len) throws IOException {
 	len += off;
 	for (int i = off; i < len ; i++)
