@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2018 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2014 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package javax.mail.internet;
@@ -81,7 +57,9 @@ public class NewsAddress extends Address {
      * @param host	the host
      */
     public NewsAddress(String newsgroup, String host) {
-	this.newsgroup = newsgroup;
+	// XXX - this method should throw an exception so we can report
+	// illegal addresses, but for now just remove whitespace
+	this.newsgroup = newsgroup.replaceAll("\\s+", "");
 	this.host = host;
     }
 
@@ -89,6 +67,7 @@ public class NewsAddress extends Address {
      * Return the type of this address.  The type of a NewsAddress
      * is "news".
      */
+    @Override
     public String getType() {
 	return "news";
     }
@@ -134,6 +113,7 @@ public class NewsAddress extends Address {
      *
      * @return		newsgroup
      */
+    @Override
     public String toString() {
 	return newsgroup;
     }
@@ -141,6 +121,7 @@ public class NewsAddress extends Address {
     /**
      * The equality operator.
      */
+    @Override
     public boolean equals(Object a) {
 	if (!(a instanceof NewsAddress))
 	    return false;
@@ -155,6 +136,7 @@ public class NewsAddress extends Address {
     /**
      * Compute a hash code for the address.
      */
+    @Override
     public int hashCode() {
 	int hash = 0;
 	if (newsgroup != null)
@@ -180,16 +162,26 @@ public class NewsAddress extends Address {
 	if (addresses == null || addresses.length == 0)
 	    return null;
 
-	StringBuffer s = 
-		new StringBuffer(((NewsAddress)addresses[0]).toString());
-	for (int i = 1; i < addresses.length; i++)
-	    s.append(",").append(((NewsAddress)addresses[i]).toString());
+	StringBuilder s =
+		new StringBuilder(((NewsAddress)addresses[0]).toString());
+	int used = s.length();
+	for (int i = 1; i < addresses.length; i++) {
+	    s.append(",");
+	    used++;
+	    String ng = ((NewsAddress)addresses[i]).toString();
+	    if (used + ng.length() > 76) {
+		s.append("\r\n\t");
+		used = 8;
+	    }
+	    s.append(ng);
+	    used += ng.length();
+	}
 	
 	return s.toString();
     }
 
     /**
-     * Parse the given comma separated sequence of newsgroup into
+     * Parse the given comma separated sequence of newsgroups into
      * NewsAddress objects.
      *
      * @param newsgroups	comma separated newsgroup string
@@ -200,7 +192,7 @@ public class NewsAddress extends Address {
 				throws AddressException {
 	// XXX - verify format of newsgroup name?
 	StringTokenizer st = new StringTokenizer(newsgroups, ",");
-	List<NewsAddress> nglist = new ArrayList<NewsAddress>();
+	List<NewsAddress> nglist = new ArrayList<>();
 	while (st.hasMoreTokens()) {
 	    String ng = st.nextToken();
 	    nglist.add(new NewsAddress(ng));
