@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 2014, 2019 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 2014 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package com.sun.mail.imap.protocol;
@@ -73,15 +49,17 @@ public class IMAPProtocolTest {
      * Test that a response containing multiple BODY elements
      * returns the correct one.  Derived from a customer bug
      * with Exchange 2003.  Normally this would never happen,
-     * but it's a valid IMAP response and JavaMail needs to
+     * but it's a valid IMAP response and Jakarta Mail needs to
      * handle it properly.
      */
     @Test
     public void testMultipleBodyResponses() throws Exception {
+	Properties props = new Properties();
+	props.setProperty("mail.imap.reusetagprefix", "true");
 	IMAPProtocol p = new IMAPProtocol(
 	    new AsciiStringInputStream(response),
 	    new PrintStream(new ByteArrayOutputStream()),
-	    new Properties(),
+	    props,
 	    debug);
 	BODY b = p.fetchBody(1, "1.1");
 	assertEquals("section number", "1.1", b.getSection());
@@ -101,10 +79,12 @@ public class IMAPProtocolTest {
      */
     @Test
     public void testMultipleBodyResponses2() throws Exception {
+	Properties props = new Properties();
+	props.setProperty("mail.imap.reusetagprefix", "true");
 	IMAPProtocol p = new IMAPProtocol(
 	    new AsciiStringInputStream(response),
 	    new PrintStream(new ByteArrayOutputStream()),
-	    new Properties(),
+	    props,
 	    debug);
 	BODY b = p.fetchBody(1, "1.1", 0, content.length(), null);
 	assertEquals("section number", "1.1", b.getSection());
