@@ -1,41 +1,17 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 1997, 2018 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 1997-2015 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
- * or packager/legal/LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at packager/legal/LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package javax.mail.internet;
@@ -61,7 +37,6 @@ import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertThat;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
-import org.junit.Ignore;
 import org.junit.Test;
 
 /**
@@ -297,7 +272,6 @@ public class MailDateFormatTest {
      * Parsing - FWS (folding white space)
      */
     @Test
-    @Ignore("until parse throws ParseException instead of returning null (1.6)")
     public void mustFailOrSkipCfws() {
         String input = "(3 Jan 2015 00:00 +0000) 1 Jan 2015 00:00 +0000";
         try {
@@ -359,17 +333,15 @@ public class MailDateFormatTest {
      * Parsing - year
      */
     @Test
-    @Ignore("until the next major release (1.6)")
-    public void lenientMustRejectSingleDigitYears() {
-        mustFail(getLenient(), "1 Jan 1 00:00 +0000", 6);
+    public void lenientMustAcceptSingleDigitYears() {
+        mustPass(getLenient(), "1 Jan 1 00:00 +0000");
     }
 
     @Test
-    @Ignore("until the next major release (1.6)")
-    public void lenientMustRejectYearsBetween1000and1899() {
+    public void lenientMustAcceptYearsBetween1000and1899() {
         mustPass(getLenient(), "1 Jan 999 00:00 +0000");
-        mustFail(getLenient(), "1 Jan 1000 00:00 +0000", 6);
-        mustFail(getLenient(), "1 Jan 1899 00:00 +0000", 6);
+        mustPass(getLenient(), "1 Jan 1000 00:00 +0000");
+        mustPass(getLenient(), "1 Jan 1899 00:00 +0000");
         mustPass(getStrict(), "1 Jan 1900 00:00 +0000");
     }
 
@@ -461,7 +433,6 @@ public class MailDateFormatTest {
     }
 
     @Test(expected = UnsupportedOperationException.class)
-    @Ignore("until the next spec change (1.6)")
     public void mustProhibitApplyLocalizedPattern() {
         SimpleDateFormat fmt = getStrict();
         fmt.applyLocalizedPattern("yyyy");
@@ -470,7 +441,6 @@ public class MailDateFormatTest {
     }
 
     @Test(expected = UnsupportedOperationException.class)
-    @Ignore("until the next spec change (1.6)")
     public void mustProhibitApplyPattern() {
         SimpleDateFormat fmt = getStrict();
         fmt.applyPattern("yyyy");
@@ -479,19 +449,16 @@ public class MailDateFormatTest {
     }
 
     @Test(expected = UnsupportedOperationException.class)
-    @Ignore("until the next spec change (1.6)")
     public void mustProhibitGet2DigitYearStart() {
         getDefault().get2DigitYearStart();
     }
 
     @Test(expected = UnsupportedOperationException.class)
-    @Ignore("until the next spec change (1.6)")
     public void mustProhibitSet2DigitYearStart() {
         getDefault().set2DigitYearStart(new Date());
     }
 
     @Test(expected = UnsupportedOperationException.class)
-    @Ignore("until the next spec change (1.6)")
     public void mustProhibitSetDateFormatSymbols() {
         SimpleDateFormat fmt = getStrict();
         fmt.setDateFormatSymbols(new DateFormatSymbols(Locale.FRENCH));
@@ -539,12 +506,8 @@ public class MailDateFormatTest {
     private void mustFail(DateFormat fmt, String input, int errorOffset) {
         try {
             Date result = fmt.parse(input);
-            // TODO remove the if statement once parse throws ParseException
-            // instead of returning null (1.6)
-            if (result != null) {
-                fail(String.format("'%s' is not a valid date in %s mode", input,
-                        fmt.isLenient() ? "lenient" : "strict"));
-            }
+            fail(String.format("'%s' is not a valid date in %s mode", input,
+                    fmt.isLenient() ? "lenient" : "strict"));
         } catch (ParseException e) {
             assertThat(e.getErrorOffset(), is(errorOffset));
         }
