@@ -1,46 +1,21 @@
 /*
- * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * Copyright (c) 2011, 2018 Oracle and/or its affiliates. All rights reserved.
  *
- * Copyright (c) 2011-2017 Oracle and/or its affiliates. All rights reserved.
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0, which is available at
+ * http://www.eclipse.org/legal/epl-2.0.
  *
- * The contents of this file are subject to the terms of either the GNU
- * General Public License Version 2 only ("GPL") or the Common Development
- * and Distribution License("CDDL") (collectively, the "License").  You
- * may not use this file except in compliance with the License.  You can
- * obtain a copy of the License at
- * https://oss.oracle.com/licenses/CDDL+GPL-1.1
- * or LICENSE.txt.  See the License for the specific
- * language governing permissions and limitations under the License.
+ * This Source Code may also be made available under the following Secondary
+ * Licenses when the conditions for such availability set forth in the
+ * Eclipse Public License v. 2.0 are satisfied: GNU General Public License,
+ * version 2 with the GNU Classpath Exception, which is available at
+ * https://www.gnu.org/software/classpath/license.html.
  *
- * When distributing the software, include this License Header Notice in each
- * file and include the License file at LICENSE.txt.
- *
- * GPL Classpath Exception:
- * Oracle designates this particular file as subject to the "Classpath"
- * exception as provided by Oracle in the GPL Version 2 section of the License
- * file that accompanied this code.
- *
- * Modifications:
- * If applicable, add the following below the License Header, with the fields
- * enclosed by brackets [] replaced by your own identifying information:
- * "Portions Copyright [year] [name of copyright owner]"
- *
- * Contributor(s):
- * If you wish your version of this file to be governed by only the CDDL or
- * only the GPL Version 2, indicate your decision by adding "[Contributor]
- * elects to include this software in this distribution under the [CDDL or GPL
- * Version 2] license."  If you don't indicate a single choice of license, a
- * recipient has the option to distribute your version of this file under
- * either the CDDL, the GPL Version 2 or to extend the choice of license to
- * its licensees as provided above.  However, if you add GPL Version 2 code
- * and therefore, elected the GPL Version 2 license, then the option applies
- * only if the new code is made subject to such option by the copyright
- * holder.
+ * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
  */
 
 package javax.mail.internet;
 
-import com.sun.mail.test.AsciiStringInputStream;
 import java.io.*;
 import java.nio.charset.Charset;
 import java.util.Properties;
@@ -49,6 +24,8 @@ import javax.activation.DataHandler;
 
 import javax.mail.*;
 
+import com.sun.mail.test.AsciiStringInputStream;
+
 import org.junit.*;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.assertFalse;
@@ -150,9 +127,7 @@ public class MimeBodyPartTest {
 	    "\n" +
 	    "\u0001\u0002\u0003" +
 	    "\n";
-	InputStream in = new ByteArrayInputStream(part.getBytes("iso-8859-1"));
-	MimeBodyPart mbp = new MimeBodyPart(in);
-	in.close();
+	MimeBodyPart mbp = new MimeBodyPart(new AsciiStringInputStream(part));
 	MimeBodyPart mbp2 = new MimeBodyPart() {
 	    @Override
 	    public void setDataHandler(DataHandler dh)
@@ -164,7 +139,7 @@ public class MimeBodyPartTest {
 	mbp2.setDataHandler(mbp.getDataHandler());
 	assertEquals("base64", mbp2.getEncoding());
 	// ensure the data is correct by reading the first byte
-	in = mbp2.getInputStream();
+	InputStream in = mbp2.getInputStream();
 	assertEquals(1, in.read());
 	in.close();
     }
@@ -179,14 +154,29 @@ public class MimeBodyPartTest {
 	    "Content-Type: application/x-test; type=a/b\n" +
 	    "\n" +
 	    "\n";
-	InputStream in = new ByteArrayInputStream(part.getBytes("iso-8859-1"));
-	MimeBodyPart mbp = new MimeBodyPart(in);
-	in.close();
+	MimeBodyPart mbp = new MimeBodyPart(new AsciiStringInputStream(part));
 	assertTrue("complete MIME type", mbp.isMimeType("application/x-test"));
 	assertTrue("pattern MIME type", mbp.isMimeType("application/*"));
 	assertFalse("wrong MIME type", mbp.isMimeType("application/test"));
     }
 
+    /**
+     * Test that a Content-Transfer-Encoding header with no value doesn't
+     * cause an IOException.
+     */
+    @Test
+    public void testEmptyContentTransferEncoding() throws Exception {
+	String part = 
+	    "Content-Type: text/plain; charset=\"us-ascii\"\n" +
+	    "Content-Transfer-Encoding: \n" +
+	    "\n" +
+	    "test" +
+	    "\n";
+	MimeBodyPart mbp = new MimeBodyPart(new AsciiStringInputStream(part));
+	assertEquals("empty C-T-E value", null, mbp.getEncoding());
+	assertEquals("empty C-T-E data", "test\n", mbp.getContent());
+    }
+
 
     private static MimeMessage createMessage(Session s)
 				throws MessagingException {
